<?php

namespace App\Presenters;

use Nette;
use Nette\Application\UI\Form;
use Nette\Application\UI;


final class PostPresenter extends Nette\Application\UI\Presenter {
    private $database;

    /** @persistent */
    public $backlink;

    /** @var \CommentForm @inject */
    public $commentFormFactory;

    public function __construct(Nette\Database\Context $database) {
        $this->database = $database;
    }

    public function renderDefault()
    {
        $this->template->posts = $this->database->table('posts');
    }

    public function beforeRender()
    {
        parent::beforeRender();
        $this->backlink = $this->storeRequest();
    }

    public function createComponentPostForm() {
        $form = new Form;
        $form->addText('title', 'Title:')
            ->setRequired();
        $form->addSelect('category', 'Category:', [
            'hw' => 'HW',
            'sw' => 'SW',
        ]);
        $form->addTextArea('text', 'Text:')
            ->setRequired();
        $form->addHidden('user_id' )->setValue($this->getUser()->id);
        $form->addSubmit('send', 'Save');
        $form->onSuccess[] = [$this, 'postFormSucceeded'];

        return $form;
    }

    public function postFormSucceeded(Form $form, \stdClass $values)
    {
        $start = microtime(true);
        if (!$this->getUser()->isLoggedIn()) {
            $this->error('You have to be log in.');
        }

        for($i = 0; $i < 100; $i++) {
            $this->database->table('posts')->insert($values);
        }
        $time = microtime(true) - $start;
        $this->flashMessage("Post created.", 'success');
        $this->redirect('default');
    }

    public function actionCreate()
    {
        if (!$this->getUser()->isLoggedIn()) {
            $this->redirect('Sign:in');
        }
    }

    protected function createComponentCommentForm($id)
    {
        $this->commentFormFactory->setId($id);
        $form = $this->commentFormFactory->create();
        $form->onSuccess[] = function (UI\Form $form) {
            $this->flashMessage("Comment added.", 'success');
            $this->restoreRequest($this->backlink);
        };

        return $form;
    }

    public function renderShow($id)
    {
        $post = $this->database->table('posts')->get($id);
        $comments = $this->database->table('comments')->where('post_id', $id);
        $this->template->post = $post;
        $this->template->comments = $comments;
    }
}
