<?php
/**
 * Created by PhpStorm.
 * User: Patrik
 * Date: 16.11.2018
 * Time: 17:51
 */

namespace App\Controller;

use App\Entity\Comment;
use App\Entity\Post;
use App\Form\CommentType;
use App\Form\PostType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;

class PostController extends AbstractController
{
    public function all() {
        $start = microtime(true);

        $db = $this->getDoctrine()->getRepository(Post::class);
        $posts = $db->findAll();


        $time = microtime(true) - $start;
        return $this->render('posts/post.html.twig', ['posts' => $posts, 'time' => $time]);
    }

    /**
     * @Route("/show/{id}", name="show_post")
     */
    public function show(Request $request, $id) {
        $this->denyAccessUnlessGranted('IS_AUTHENTICATED_FULLY'); // user is logged in

        $entityManager = $this->getDoctrine()->getManager();
        $post = $entityManager->getRepository(Post::class)->find($id);
        $comments = $entityManager->getRepository(Comment::class)->findBy(['post' => $id]);


        //create new comment
        $comment = new Comment();
        $form = $this->createFormBuilder($comment)
            ->setAction($this->generateUrl('create_comment'))
            ->setMethod('POST')
            ->add('text', TextType::class, ['attr' => ['class' => 'form-control']])
            ->add('id', HiddenType::class, array(
                'data' => $id))
            ->add('Send', SubmitType::class, ['attr' => ['class' => 'btn ']])
            ->getForm();

        return $this->render('posts/show.html.twig', ['post' => $post,
            'comments' => $comments,
            'form' => $form->createView()]);
    }

    /**
     * @Route("/create", name="create_post")
     */
    public function create(Request $request) {
        //form
        $post = new Post();
        $form = $this->createForm(PostType::class, $post);

        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            //save record
            $post->setUser($this->getUser());
            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->persist($post);
            $entityManager->flush();

            //message
            $this->addFlash(
                'success',
                'Post created!'
            );

            //redirect
            return $this->redirectToRoute('index');
        }

        return $this->render('posts/create.html.twig', ['form' => $form->createView()
        ]);
    }
}